// Classe qui gre un dgrad de couleurs.
// Auteur : Jrme Conus
#include "colorgradient.h"

// Construit un dgrad de couleurs.
ColorGradient::ColorGradient()
{
    clear();
}

// Efface toutes les couleurs cl et rtabli le dgrad de base : du noir
// au blanc.
void ColorGradient::clear() {
    m_ColorGradient.clear();
    setKey(0, Qt::black);
    setKey(1, Qt::white);

}

// Ajoute une couleur cl  la position donne.
void ColorGradient::setKey(float Position, const QColor& rColor) {
    m_ColorGradient[Position] = rColor;
}

// Retourne la couleur de la position donne.
QColor ColorGradient::colorAt(float Position) const {
    // Si ncessaire, ajuste la position pour qu'elle soit dans l'intervalle [0,1]
    if (Position > 1.)
        Position -= static_cast<int>(Position);
    else if (Position < 0.)
        Position = static_cast<int>(Position) - Position;

    // Recherche d'une cl correspondant  la position
    if (m_ColorGradient.contains(Position))
        return m_ColorGradient.value(Position);

    // Recherche de la valeur du dgrad
    float LeftPosition = 0.;
    float RightPosition = 1.;

    QMap<float, QColor>::ConstIterator Key = m_ColorGradient.constBegin();
    for (; Key != m_ColorGradient.constEnd(); ++Key) {
        if (Key.key() < Position)
            LeftPosition = Key.key();

        if (Key.key() > Position && Key.key() < RightPosition)
            RightPosition = Key.key();
    }

    QColor LeftColor = m_ColorGradient.value(LeftPosition);
    QColor RightColor = m_ColorGradient.value(RightPosition);

    float SpectrumWidth = RightPosition - LeftPosition;

    int Red = (RightColor.red() - LeftColor.red()) * (Position - LeftPosition) / SpectrumWidth + LeftColor.red();
    int Green = (RightColor.green() - LeftColor.green()) * (Position - LeftPosition) / SpectrumWidth + LeftColor.green();
    int Blue = (RightColor.blue() - LeftColor.blue()) * (Position - LeftPosition) / SpectrumWidth + LeftColor.blue();

    return QColor(Red, Green, Blue);
}
